open Unix
open Pervasives

open Conf
open Types
open Utils

(**************************************************************)
(* understand *)
(**************************************************************)

(* Try to understand what we received *)
let rec compute_read conn =
  match conn.state_ans with
  | WAIT_DNS _ ->
      failwith "wait_dns in Answer.compute_read"
  | CMD_LINE ->
      (try
        let pos = String.index conn.read_ans '\n' in
        conn.write_ans <- String.sub conn.read_ans 0 (pos+1);
        conn.read_ans <-
          String.sub conn.read_ans (pos+1)
            ((String.length conn.read_ans)-pos-1);
        conn.state_ans <- HEADERS;
        if !Types.ans_1 then
          (print_string conn.write_ans; flush stdout);
        compute_read conn
      with Not_found ->
        (* the command line is not finished *)
        ())
  | HEADERS ->
      (try
        let pos = String.index conn.read_ans '\n' in
        let header = String.sub conn.read_ans 0 (pos+1) in
        conn.read_ans <-
          String.sub conn.read_ans (pos+1)
            ((String.length conn.read_ans)-pos-1);
        if pos <= 2 then
          (* last line of headers *)
          (if !Types.ans_2 then
            (print_string header; flush stdout);
           conn.state_ans <- CONTENT;
           conn.write_ans <- conn.write_ans ^ header)
        else
          (if ok_header header then
            (if !Types.ans_2 then
              (print_string header; flush stdout);
             conn.write_ans <- conn.write_ans ^ header));
        compute_read conn
      with Not_found ->
        (* this line of headers is not finished *)
        ())
  | CONTENT ->
      if !Types.ans_3 then
        (print_string conn.read_ans; flush stdout);
      conn.write_ans <- conn.write_ans ^ conn.read_ans;
      conn.read_ans <- ""

(**************************************************************)
(* read and write *)
(**************************************************************)

(* buffer used for read *)
let buf = String.create buf_size

(* Update every connexions *)
let gere_conns active_read active_write conns =

  let rec gere_aux conns = function
    | [] -> conns
    | conn::l ->
        match conn.state with
        | STARTING
        | DNS _
        | CONNECTING ->
            (* nothing yet to read *)
            gere_aux (conn::conns) l
        | ALIVE ->
            if List.exists ((=) conn.server) active_read then
              (let cont = manage_read conn in
              if conn.read_ans <> "" then compute_read conn;
              if not cont then finish conn "";
              manage_write conn conns l)
            else
              (* try to write what must be *)
              manage_write conn conns l
        | FINISHING ->
            (* we just have to finish writing data  to the client *)
            manage_write conn conns l

  (* write what can be and is ready *)
  and manage_write conn conns l =
    let fin = ref false in
    (try
      let nb = Unix.write conn.client conn.write_ans 0
          (String.length conn.write_ans) in
      conn.write_ans <- String.sub conn.write_ans nb
          ((String.length conn.write_ans) - nb)
    with
    | Unix_error (EAGAIN, "write", "") ->
        ()
    | Unix_error (_, "write", "") ->
        fin := true);
    if !fin then
      (* Connexion closed by client *)
      (close_connexion conn;
       gere_aux conns l)
    else
      (* Unable to write any more now *)
      if conn.state = FINISHING && conn.write_ans = "" then
        (* That's finished... *)
        (close_connexion conn;
         gere_aux conns l)
      else
        (if conn.write_ans <> "" then
          fwrite := conn.client :: !fwrite;
         gere_aux (conn::conns) l)

  (* read all available data
   * return true if the connexion is still alive
   *        false if the connexion is closed *)
  and manage_read conn =
    let cont = ref true in
    let ok = ref true in
    (try
      while !cont do
        let nb = Unix.read conn.server buf 0 buf_size in
        conn.read_ans <- conn.read_ans ^ (String.sub buf 0 nb);
        match nb with
        | 0 -> (* connexion closed *)
            cont := false;
            ok := false
        | x when x=buf_size -> (* go on *)
            ()
        | _ -> (* nothing more to read now *)
            cont := false
      done
    with
    | Unix_error (EAGAIN, "read", "") -> (* Nothing to read now *)
        ()
    | Unix_error (_, "read", "") -> (* connexion closed by SIGPIPE *)
        ok := false);
    !ok
  in

  gere_aux [] conns
