open Conf
open Types
open Utils

(* lecture des arguments de la ligne de commande *)
let conf_file = ref "swift.conf"

let usage = "usage: " ^ Sys.argv.(0) ^ " [-c config_file]"
let speclist =
  [("-c", Arg.String (fun c -> conf_file:=c),
    ": name of the configuration file (default : swift.conf)")]

(* Main loop of the program *)
let schedule fd_serv conns =
  if !Types.debug > 1 then
    (print_int (List.length !conns);
     print_string ", ";
     print_int (List.length !fread);
     flush stdout);
  let active_read, active_write, _ =
    Unix.select (!Types.from_dns_fds::fd_serv::!fread) !fwrite [] (-.1.) in
  if !Types.debug > 1 then
    (print_string ", ";
     print_int (List.length active_read);
     print_string ", ";
     print_int (List.length active_write);
     print_string "\n";
     flush stdout);
  fwrite := [];
  (* Are there new connexions *)
  if List.exists ((=) fd_serv) active_read then
    (let fds, addr = Unix.accept fd_serv in
    Unix.set_nonblock fds;
    conns := { client = fds;
               server = Unix.stdin;
               state = STARTING;
               read_req = ""; write_req = ""; state_ans = CMD_LINE;
               read_ans = ""; write_ans = ""; state_req = CMD_LINE }::!conns;
    fread := fds::!fread);
  (* Read and write what can be *)
  Dns.update active_read;
  conns := Request.gere_conns active_read active_write !conns;
  conns := Answer.gere_conns active_read active_write !conns

let main () =
  (* Read the arguments *)
  Arg.parse
    speclist
    (fun  x -> raise (Arg.Bad ("Bad argument : " ^ x)))
    usage;
  (* read configuration file *)
  Conf.init !conf_file;
  (* ignore sigpipe *)
  Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
  (* Bind the socket *)
  let fd_serv = bind_server !Conf.port in
  let conns = ref [] in
  if (!Types.debug + !Types.stats) > 0 then
    (print_string "Swift is waiting for request\n"; flush stdout);
  while true do
    schedule fd_serv conns
  done;;

main ()
