open Types

(****************************)
(* Operations on connexions *)
(****************************)

(* bind the server socket *)
let bind_server port =
  let fds = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
  Unix.setsockopt fds Unix.SO_REUSEADDR true;
  Unix.bind fds (Unix.ADDR_INET (Unix.inet_addr_any, port));
  Unix.listen fds 5;
  fds

(* catch all exceptions made by a function *)
let trywith f =
  try
    f ()
  with _ -> ()

(* Closes a connexion *)
let close_connexion conn =
  fread := List.filter ((<>) conn.client) !fread;
  trywith
    (fun () -> Unix.shutdown conn.client Unix.SHUTDOWN_ALL);
  trywith
    (fun () -> Unix.close conn.client);
  if conn.state <> FINISHING && conn.state <> STARTING then
    (fread := List.filter ((<>) conn.server) !fread;
     trywith
       (fun () -> Unix.shutdown conn.server Unix.SHUTDOWN_ALL);
     trywith
       (fun () -> Unix.close conn.server))

(* put a connexion in a state where it just has to
 * finish writing data to the client and then exit *)
let finish conn str =
  if str <> "" then
    conn.write_ans <- str;
  if conn.state <> FINISHING && conn.state <> STARTING then
    (fread := List.filter ((<>) conn.server) !fread;
     trywith
       (fun () -> Unix.shutdown conn.server Unix.SHUTDOWN_ALL);
     trywith
       (fun () -> Unix.close conn.server));
  conn.state <- FINISHING

(***********************)
(* Operations on files *)
(***********************)

(* really read nb bytes on a fds *)
let really_read fds nb =
  let ans = String.create nb in
  let nb_read = ref 0 in
  while !nb_read < nb do
    nb_read := !nb_read + (Unix.read fds ans !nb_read (nb - !nb_read))
  done;
  ans

(* read a whole file *)
let read_file name =
  let chan = open_in_bin name in
  let buf_size = ref 1024 in
  let buf = ref (String.create !buf_size) in
  let nb_read = ref 0 in
  let cont = ref true in
  while !cont do
    let tmp = input chan !buf !nb_read (!buf_size - !nb_read) in
    if tmp = 0
    then cont := false
    else
      (nb_read := !nb_read + tmp;
       if !nb_read = !buf_size
       then
         (buf := !buf ^ (String.create !buf_size);
          buf_size := 2 * !buf_size))
  done;
  String.sub !buf 0 !nb_read

(* Give the next token of a channel
 * Commentaries go from # to the end of the line  
 * this is used for reading the configuration file
 *)
let next_token chan =
  let res = ref "" in
  (try
    let ch = ref (input_char chan) in
    while !ch = ' ' || !ch = '\n' || !ch = '\r'
          || !ch = '\t' || !ch = '#' do
      if !ch = '#'
      then (let _ = input_line chan in ());
      ch := input_char chan
    done;
    while not (!ch = ' ' || !ch = '\n' || !ch = '\r'
               || !ch = '\t' || !ch = '#') do
      res := !res ^ (String.make 1 !ch);
      ch := input_char chan
    done;
  with _ -> ());
  !res

(* does the string match the pattern *)
let match_pattern str pat =
  let size_s = String.length str in
  let size_p = String.length pat in
  let rec match_aux pos_s pos_p =
    if pos_p = size_p then
      pos_s = size_s
    else
      ((pat.[pos_p] = '*' && match_aux pos_s (pos_p+1))
     ||
       (pos_s <> size_s &&
        (match pat.[pos_p] with
        | '*' ->
            match_aux (pos_s+1) pos_p
        | '?' -> match_aux (pos_s+1) (pos_p+1)
        | x ->
            x=str.[pos_s] && match_aux (pos_s+1) (pos_p+1))))
  in
  match_aux 0 0
